@@ -130,7 +130,15 @@ def warn(msg):
     """Emit NCrystalUserWarning via standard warnings.warn function"""
     from .exceptions import NCrystalUserWarning
     import warnings
-    warnings.warn( NCrystalUserWarning(str(msg)), stacklevel = 2 )
+    m = str(msg)
+    if _add_warn_counts_to_msgs[0]:
+        _add_warn_counts_to_msgs[1] += 1
+        m = '%s [warn#%i]'%(m,_add_warn_counts_to_msgs[1])
+    warnings.warn( NCrystalUserWarning(m), stacklevel = 2 )
+
+#Hook to avoid repeated warnings to be silenced during unit tests, by appending
+#a warning number to them:
+_add_warn_counts_to_msgs = [False,0]
 
 class WarningSpy:
     """Context manager which spies on any warnings emitted via warnings
@@ -420,14 +428,23 @@ def extract_path( s ):
             return res_try
         return pathlib.Path(s)
 
-def download_url( url, decode_as_utf8_str = True, wrap_exception = True ):
+def download_url( url,
+                  decode_as_utf8_str = True,
+                  wrap_exception = True,
+                  timeout = None,
+                  quiet_network_fail = False ):
+    """Download the provided url. The timeout value is in seconds if given.
+    Returns None on network failure if quiet_network_fail is True.
+    """
     import urllib.request
     import urllib.error
     try:
         req = urllib.request.Request(url)
-        with urllib.request.urlopen(req) as response:
+        with urllib.request.urlopen(req, timeout = timeout ) as response:
             data = response.read()
     except urllib.error.URLError as e:
+        if quiet_network_fail:
+            return None
         if wrap_exception:
             from .exceptions import NCException
             raise NCException(f'Error downloading url "{url}": {e}')
