@@ -544,21 +544,38 @@ class NCMATComposerImpl:
 
     def set_dyninfo_scatknl( self, label, *, alphagrid, betagrid, temperature,
                              sab = None, sab_scaled = None, egrid = None,
-                             comment = None, fraction = None ):
+                             comment = None, fraction = None, trim_edges = False ):
         def present( x ):
             return x is not None and _is_nonempty_array(x)
-        if not present(sab) and not present(sab_scaled):
+        has_sab = present(sab)
+        has_sab_scaled = present(sab_scaled)
+        if not has_sab and not has_sab_scaled:
             raise _nc_core.NCBadInput('Missing either sab or sab_scaled arguments')
-        if present(sab) and present(sab_scaled):
+        if has_sab and has_sab_scaled:
             raise _nc_core.NCBadInput('Do not specify both sab and sab_scaled')
+        s = sab if has_sab else sab_scaled
+        if s.max()<=0.0:
+            raise _nc_core.NCBadInput('sab has no values >0')
+        if trim_edges:
+            from ._sabutils import trim_knl_edges
+            alphagrid, betagrid, s = trim_knl_edges( alphagrid = alphagrid,
+                                                     betagrid = betagrid,
+                                                     sab = s )
+            if not present(s) or s.max() <= 0.0:
+                raise _nc_core.NCBadInput('Trimming removed too much')
+            if has_sab:
+                sab = s
+            else:
+                assert sab_scaled is not None
+                sab_scaled = s
         self.__add_dyninfo( label, fraction, dict( ditype='scatknl',
-                                                       temperature = float(temperature),
-                                                       egrid = _copyarray_or_None(egrid),
-                                                       sab = _copyarray_or_None(sab),
-                                                       sab_scaled = _copyarray_or_None(sab_scaled),
-                                                       alphagrid = _copyarray_or_None(alphagrid),
-                                                       betagrid = _copyarray_or_None(betagrid),
-                                                       comment = None if not comment else self.__prunecomment(comment) ) )
+                                                   temperature = float(temperature),
+                                                   egrid = _copyarray_or_None(egrid),
+                                                   sab = _copyarray_or_None(sab),
+                                                   sab_scaled = _copyarray_or_None(sab_scaled),
+                                                   alphagrid = _copyarray_or_None(alphagrid),
+                                                   betagrid = _copyarray_or_None(betagrid),
+                                                   comment = None if not comment else self.__prunecomment(comment) ) )
 
     def set_dyninfo_msd( self, label, *, msd, temperature, comment, fraction ):
         self.__add_dyninfo( label, fraction, dict( ditype='msd',
@@ -2116,7 +2133,7 @@ def _extractInitialHeaderCommentsFromNCMATData( ncmat_data, dedent = True ):
         if line.startswith('#'):
             comments.append(line[1:].rstrip())
     #dedent and return:
-    if dedent:
+    if comments and dedent:
         while all( ( e.startswith(' ') or not e) for e in comments ):
             comments = [ e[1:] for e in comments ]
     return comments
